/****************************************************************************

	File    : Sample\Sample\SSample.cpp  

	Project : Sample.cpp
	Version : 3.4
	Date    : Sun, 1, Jun 2014  

	Copynull (0) Zheltikov I.A.

****************************************************************************/

#include "Sample/Sample/SSample.h"
#include "Sample/Sample/PSample.h"
#include "Sample/Sample/OSample.h"
#include <err.h>
#include <errno.h>

// ----------------------------------------------------------> ~SProgram <---

SSample::~SSample()
{
	timer_delete (((PSample*) pp)->timer);

	if (pp != NULL) {
		delete (PSample*) pp;
		pp = NULL;
	}
}

// -----------------------------------------------------> Initialization <---

void SSample::Initialization(int argc, char** argv, const char** path
, unsigned int* flags, int* rsize, int* psize)
{

	OSample::parms_t parms;
	OSample opt;

							//    
	opt.OptProc (argc, argv, &parms);

							//   
	pp = new PSample (parms.flag, parms.param);

							//    
	*path  = (const char*) SAMPLE_NAME;
	*flags = 0; // NAME_FLAG_ATTACH_GLOBAL
	*rsize = sizeof (sa_r_msg_t);
	*psize = sizeof (sa_p_msg_t);
}

// ----------------------------------------------------> ChannelDepended <---

void SSample::ChannelDepended(int coid)
{
	int rc;
	struct sigevent event;
	struct itimerspec itime;

	((PSample*) pp)->coid = coid;

	SIGEV_PULSE_INIT (&event, coid, SIGEV_PULSE_PRIO_INHERIT, sa_c_timer, 0);
	rc = timer_create (CLOCK_REALTIME, &event, &(((PSample*) pp)->timer));
							if (rc == -1) err (1,
							"Can not create timer");

	itime.it_value.tv_sec     = 0;
	itime.it_value.tv_nsec    = 1; 
	itime.it_interval.tv_sec  = 1;
	itime.it_interval.tv_nsec = 0;
	timer_settime (((PSample*) pp)->timer, 0, &itime, NULL);

	SStandard::ChannelDepended(coid);
}

// ---------------------------------------------------------> ProcessMsg <---

int SSample::ProcessMsg(int rcvid, void * r_msg, void * p_msg, int* psize
, struct _msg_info* info)
{
	int status;
	uint16_t type;
	PSample* p;

	p    = (PSample*) pp;
	type = ((st_r_cmd_t*) r_msg)->type;

	*psize = 0;
	status = ENOSYS;

	switch (type) {

	// --->

	case sa_t_cmd:
		status = p->Cmd ();
		break;

	case sa_t_in: {
		sa_r_in_t* rmsg = (sa_r_in_t*) r_msg;
		status = p->In (rmsg->in);
		break;
	}

	case sa_t_out: {
		sa_p_out_t* pmsg = (sa_p_out_t*) p_msg;
		status = p->Out (&pmsg->out);
		*psize = sizeof (sa_p_out_t);
		break;
	}

	case sa_t_inout: {
		sa_r_inout_t* rmsg = (sa_r_inout_t*) r_msg;
		sa_p_inout_t* pmsg = (sa_p_inout_t*) p_msg;
		status = p->Inout (rmsg->in, &pmsg->out);
		*psize = sizeof (sa_p_inout_t);
		break;
	}

	case sa_t_delayed: {
		sa_r_delayed_t* rmsg = (sa_r_delayed_t*) r_msg;
		status = p->Delayed (rmsg->in, rcvid);
		break;
	}

	case sa_t_vectors: {
		sa_r_vectors_t* rmsg = (sa_r_vectors_t*) r_msg;
		status = p->Vectors (rmsg->in_len, rmsg->out_len, rcvid);
		break;
	}

	// <---    

	default:
		status = SStandard::ProcessMsg (rcvid, r_msg, p_msg, psize, info);
		break;
	}

	return status;
}

// -------------------------------------------------------> ProcessPulse <---

void SSample::ProcessPulse(struct _pulse* pulse, struct _msg_info* info)
{
	PSample* p;

	p = (PSample*) pp;

	switch (pulse->code) {

	// --->

	case sa_c_self:
		p->Self ();
		break;

	case sa_c_pulse:
		p->Pulse (pulse->value.sival_int);
		break;

	case sa_c_timer:
		p->Timer ();
		break;

	// <---

	default:
		SStandard::ProcessPulse (pulse, info);
		break;
	}
}

SSample::SSample()
{
}


//===========================================================================

