/****************************************************************************

	File    : Server\ServerFix\ServerFix.cpp  

	Project : Sample.cpp
	Version : 3.4
	Date    : Sun, 1, Jun 2014  

	Copynull (0) Zheltikov I.A.

****************************************************************************/

#include "Server/ServerFix/ServerFix.h"
#include <err.h>
#include <stdlib.h>


/*
    -   , 
 .      .
 Run()        main().
      Prepare(), 
 MainLoop(),       .
*/



ServerFix::ServerFix()
{
	is_srr_running = true;
	path = NULL;
	flags = 0;
	r_msg = NULL;
	p_msg = NULL;
	attach = NULL;
	coid = -1;
}

// ---------------------------------------------------------> ~ServerFix <---

ServerFix::~ServerFix()
{
	if (coid != -1)
		ConnectDetach (coid);

	if (attach != NULL)
		name_detach (attach, flags);

	if (p_msg != NULL)
		free (p_msg);

	if (r_msg != NULL)
		free (r_msg);
}

// ----------------------------------------------------------------> Run <---

void ServerFix::Run(int argc, char** argv)
{
	/*
	   -   main().
	*/

	int rsize;
	int psize;
	int chid;

	Initialization (argc, argv, &path, &flags, &rsize, &psize);
	Prepare (path, flags, rsize, psize, &r_msg, &p_msg, &coid, &chid);
	ChannelDepended (coid);
	MainLoop (r_msg, p_msg, chid, rsize);
}

// ------------------------------------------------------------> Prepare <---

void ServerFix::Prepare(const char* path, unsigned int flags, int rsize, int
 psize, void ** r_msg, void ** p_msg, int* coid, int* chid)
{

	/*
	    -.
	      .
	 ,     .
	*/

	*r_msg = malloc (rsize);
							if (*r_msg == NULL) err (EXIT_FAILURE,
							"Can not allocate memory for a message");

	*p_msg = malloc (psize);
							if (*p_msg == NULL) err (EXIT_FAILURE,
							"Can not allocate memory for a message");

	attach = name_attach (NULL, path, flags);
							if (attach == NULL) err (EXIT_FAILURE,
							"Can not attach the name '%s'", path);

	*coid = ConnectAttach (0, 0, attach->chid, 0, 0);
							if (*coid == -1) err (EXIT_FAILURE,
							"Can not establish a connection to self");

	*chid = attach->chid;
}

// -----------------------------------------------------------> MainLoop <---

void ServerFix::MainLoop(void * r_msg, void * p_msg, int chid, int rsize)
{
	/*
	   -.
	 ,   .
	 ,   .
	*/

	int rc;
	int rcvid;
	int psize;
	int status;
	struct _msg_info info;

	while (is_srr_running) {
		rcvid = MsgReceive (chid, r_msg, rsize, &info);
							if (rcvid == -1) {
								warn ("MsgReceive error");
								continue;
							}

		if (rcvid != 0) {
			status = ProcessMsg (rcvid, r_msg, p_msg, &psize, &info);
			if (status != -1) {
				rc = MsgReply (rcvid, status, p_msg, psize);
							if (rc == -1)
								warn ("MsgReply error");
			}
		} else {
			ProcessPulse ((struct _pulse*) r_msg, &info);
		}
	}
}


//===========================================================================

