/****************************************************************************

	File    : Server\Standard\SStandard.cpp  

	Project : Sample.cpp
	Version : 3.4
	Date    : Sun, 1, Jun 2014  

	Copynull (0) Zheltikov I.A.

****************************************************************************/

#include "Server/Standard/SStandard.h"
#include "Server/Standard/PStandard.h"
#include "Server/Standard/OStandard.h"
#include <err.h>
#include <errno.h>

SStandard::SStandard()
{
	pp = NULL;
}

// ---------------------------------------------------------> ~SStandard <---

SStandard::~SStandard()
{
	if (pp != NULL) {
		delete (PStandard*) pp;
		pp = NULL;
	}
}

// -----------------------------------------------------> Initialization <---

void SStandard::Initialization(int argc, char** argv, const char** path
, unsigned int* flags, int* rsize, int* psize)
{

	OStandard::parms_t parms;
	OStandard opt;

							//    
	opt.OptProc (argc, argv, &parms);

							//   
	pp = new PStandard ();

							//    
	*path  = (const char*) parms.name;
	*flags = parms.flag;
	*rsize = sizeof (st_r_msg_t);
	*psize = sizeof (st_p_msg_t);
}

// ----------------------------------------------------> ChannelDepended <---

void SStandard::ChannelDepended(int coid)
{
}

// ---------------------------------------------------------> ProcessMsg <---

int SStandard::ProcessMsg(int rcvid, void * r_msg, void * p_msg, int* psize
, struct _msg_info* info)
{
	int status;
	uint16_t type;
	PStandard* p;

	p    = (PStandard*) pp;
	type = ((st_r_cmd_t*) r_msg)->type;

	*psize = 0;
	status = ENOSYS;

	switch (type) {

	case st_t_quit:
		status = p->Complete (&is_srr_running);
		break;

	case st_t_evdlvr: {
		st_r_evdlvr_t* rmsg = (st_r_evdlvr_t*) r_msg;
		status = p->EventForDelivering (rcvid, &(rmsg->event));
		break;
	}

	case _IO_CONNECT:
	case _IO_CLOSE:
		status = EOK;
		break;

	default:
		warnx ("unsupported command type=%d", type);
		status = ENOSYS;
		break;
	}

	return status;
}

// -------------------------------------------------------> ProcessPulse <---

void SStandard::ProcessPulse(struct _pulse* pulse, struct _msg_info* info)
{
	PStandard* p;

	p = (PStandard*) pp;

	switch (pulse->code) {

	case st_c_quit:
		p->CompleteByPulse (&is_srr_running);
		break;

	case _PULSE_CODE_DISCONNECT:
		ConnectDetach (pulse->scoid);
		break;

	case _PULSE_CODE_UNBLOCK:
	case _PULSE_CODE_THREADDEATH:
	case _PULSE_CODE_COIDDEATH:
	case _PULSE_CODE_NET_ACK:
	case _PULSE_CODE_NET_UNBLOCK:
	case _PULSE_CODE_NET_DETACH:
		break;

	default:
		warnx ("unsupported pulse code=%d", pulse->code);
		break;
	}
}


//===========================================================================

